#!/bin/sh

set -v

apk add git curl jq helm;
# Intentionally does not handle renames, which would begin with `R`.
changed_charts=$(git show --name-status --pretty="format:" | grep '^[MA]' | cut -f 2- | grep '^charts' | cut -d '/' -f 1,2 | sort | uniq);
echo "$changed_charts";
current_versions=$(curl -s --user scubbo:$GITEA_PASSWORD https://gitea.scubbo.org/api/v1/packages/scubbo | jq -r '.[] | select(.type == "helm") | .name + "___" + .version');
echo "$current_versions";

mkdir bundles;
for changed_chart in $changed_charts
do
  echo "Operating on $changed_chart";
  # e.g. /Users/scubbo/Code/helm-charts/hello-world-0.1.1.tgz
  output_location=$(helm package $changed_chart | rev | cut -d ' ' -f 1 | rev);
  # e.g. hello-world-0.1.1
  file_metadata=$(echo $output_location | rev | cut -d '/' -f 1 | cut -c 5- | rev);
  chart_name=$(echo $file_metadata | rev | cut -d '-' -f 2- | rev);
  chart_version=$(echo $file_metadata | rev | cut -d '-' -f 1 | rev);
  search_string="$chart_name""___""$chart_version";
  if [[ $(echo $current_versions | grep grep "$search_string" | wc -l ) -eq 1 ]]; then
    # I.e. if the current version in the repo is the same as what we're trying to push
    # TODO - should we check that we're pushing a _newer_ version?
    # TODO - does the API only show the current version, or all versions?
    echo "Version clash ($chart_version) for $chart_name";
    exit 1;
  else
    # Move to a directory so we can upload later. Do this after checking all packages so there's no partial update
    mv $output_location bundles/;
  fi
done

# All packages have been checked, no version conflicts - upload them all!
find bundles -type f -exec curl -s --user "scubbo:$GITEA_PASSWORD" -X POST --upload-file '{}' https://gitea.scubbo.org/api/packages/scubbo/helm/api/charts \;
rm -rf bundles;

# Call Argo UI to create/sync the Argo application.
#
# Prefer this over declarative GitOps - "upload a file like [here](https://argo-cd.readthedocs.io/en/stable/user-guide/helm/)
# stating which version should be used" - because that would require two changes for one version (one to declare and
# upload the new version, another to depend on it). Since we _own_ the chart definitions, there's no reason not to
# deploy the latest one immediately!
#
# An alternative approach (that would only have required a single update) would be for the Argo Application to track a
# Git Repo (at HEAD) rather than a Helm repo. In this case, merely pushing a new instance to HEAD would be sufficient to
# update the Argo App. However, I wanted to learn how to operate a Helm repo :)
for changed_chart in $changed_charts
do
  app_name=$(basename $changed_chart)
  # TODO - create app if it doesn't already exist
  curl -s -X POST "argo-cd-argocd-server.argo.svc.cluster.local/api/v1/applications/$app_name/sync" -H "Authorization: Bearer ${ARGO_TOKEN}" -H "Content-Type: application/json"
done
